==> tuneflow-plugin-basic-0.26.6.tgz: package/src/create_empty_track_plugin.ts <==
import { AudioPlugin, TrackType, TuneflowPlugin, WidgetType } from 'tuneflow';
import type {
  InputNumberWidgetConfig,
  LabelText,
  ParamDescriptor,
  Song,
  SongAccess,
} from 'tuneflow';

export class CreateEmptyTrack extends TuneflowPlugin {
  static providerId(): string {
    return 'andantei';
  }

  static pluginId(): string {
    return 'create-empty-track';
  }

  static providerDisplayName(): LabelText {
    return {
      zh: 'Andantei行板',
      en: 'Andantei',
    };
  }

  static pluginDisplayName(): LabelText {
    return {
      zh: '创建空白轨道',
      en: 'Create Empty Track',
    };
  }

  static allowReset(): boolean {
    return false;
  }

  // TODO: Support different types of track.
  params(): { [paramName: string]: ParamDescriptor } {
    return {
      insertIndex: {
        displayName: {
          zh: '插入位置',
          en: 'Insert Position',
        },
        defaultValue: -1,
        widget: {
          type: WidgetType.InputNumber,
          config: {} as InputNumberWidgetConfig,
        },
        optional: true,
        hidden: true,
      },
      type: {
        displayName: {
          zh: '轨道类型',
          en: 'Track Type',
        },
        defaultValue: undefined,
        widget: {
          type: WidgetType.None,
        },
        hidden: true,
      },
    };
  }

  songAccess(): SongAccess {
    return {
      createTrack: true,
    };
  }

  async run(song: Song, params: { [paramName: string]: any }): Promise<void> {
    const type = this.getParam<number>(params, 'type');
    const insertIndex = this.getParam<number>(params, 'insertIndex');
    const newTrack = song.createTrack({ type, index: insertIndex });
    if (type === TrackType.MIDI_TRACK) {
      newTrack.setSamplerPlugin(AudioPlugin.DEFAULT_SYNTH);
    }
  }
}

==> tuneflow-plugin-basic-0.31.0.tgz: package/src/create_empty_track_plugin.ts <==
import { AudioPlugin, TrackType, TuneflowPlugin, WidgetType } from 'tuneflow';
import type {
  InputNumberWidgetConfig,
  LabelText,
  ParamDescriptor,
  Song,
  SongAccess,
} from 'tuneflow';

export class CreateEmptyTrack extends TuneflowPlugin {
  static providerId(): string {
    return 'andantei';
  }

  static pluginId(): string {
    return 'create-empty-track';
  }

  static providerDisplayName(): LabelText {
    return {
      zh: 'Andantei行板',
      en: 'Andantei',
    };
  }

  static pluginDisplayName(): LabelText {
    return {
      zh: '创建空白轨道',
      en: 'Create Empty Track',
    };
  }

  static allowReset(): boolean {
    return false;
  }

  // TODO: Support different types of track.
  params(): { [paramName: string]: ParamDescriptor } {
    return {
      insertIndex: {
        displayName: {
          zh: '插入位置',
          en: 'Insert Position',
        },
        defaultValue: -1,
        widget: {
          type: WidgetType.InputNumber,
          config: {} as InputNumberWidgetConfig,
        },
        optional: true,
        hidden: true,
      },
      type: {
        displayName: {
          zh: '轨道类型',
          en: 'Track Type',
        },
        defaultValue: undefined,
        widget: {
          type: WidgetType.None,
        },
        hidden: true,
      },
    };
  }

  songAccess(): SongAccess {
    return {
      createTrack: true,
    };
  }

  async run(song: Song, params: { [paramName: string]: any }): Promise<void> {
    const type = this.getParam<number>(params, 'type');
    const insertIndex = this.getParam<number>(params, 'insertIndex');
    const newTrack = song.createTrack({
      type,
      index: insertIndex,
      assignDefaultSamplerPlugin: true,
    });
    if (type === TrackType.MIDI_TRACK) {
      newTrack.setSamplerPlugin(newTrack.createAudioPlugin(AudioPlugin.DEFAULT_SYNTH_TFID));
    }
  }
}
